package com.gct.warning.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gct.common.core.result.Result;
import com.gct.warning.entity.WnGkMsg;
import com.gct.warning.feignclient.BaseDataClient;
import com.gct.warning.feignclient.DgnsDataClient;
import com.gct.warning.service.WnGkMsgService;
import com.gct.warning.service.WnWellStatusService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author xusirui 2021/5/7
 */
@Service
public class MainServiceImpl {

    @Autowired
    WnWellStatusService wnWellStatusService;

    @Autowired
    WnGkMsgService wnGkMsgService;

    @Autowired
    BaseDataClient baseDataClient;

    @Autowired
    DgnsDataClient dgnsDataClient;

    public Result getMsgCount(String date, List<String> rank, Integer level, Integer orgId) {

        JSONObject jo = wnWellStatusService.getWellDataCount(date, level, orgId);

        if (jo == null) return Result.error("无数据");
        JSONArray wellDataCount = jo.getJSONArray("data");

        if (wellDataCount != null) {
            wellDataCount = wnGkMsgService.getWnGkMsgCount(date, level, rank, wellDataCount);  //在上次查询的结果中追加数据 无需分页
            //todo :工况报警

            for (int i = 0; i < wellDataCount.size(); i++) {
                JSONObject jao = wellDataCount.getJSONObject(i);
                jao.put("LYZS", 10086);
                jao.put("LYWCL", 10086);
                jao.put("HJZS", jao.getInteger("LYZS") + jao.getInteger("GKZS"));
                jao.put("HJWCL", jao.getInteger("LYWCL") + jao.getInteger("GKWCL"));
            }
        }
        return Result.ok(jo);
    }

    public Result getAllGKWarning(String date, List<String> rank, Integer level, Integer orgId, Integer curPage, Integer pageSize) {
        JSONObject jo = wnGkMsgService.getAllGKWarning(date, level, rank, orgId, curPage, pageSize);
        return jo.get("data") == null ? Result.error("无数据") : Result.ok(jo);
    }

    public Result getNotConfirmGKWarning(String date, List<String> rank, Integer level, Integer orgId, Integer curPage, Integer pageSize) {
        JSONObject jo = wnGkMsgService.getNotConfirmGKWarning(date, level, rank, orgId, curPage, pageSize);
        return jo.get("data") == null ? Result.error("无数据") : Result.ok(jo);
    }

    public Result getProblemListItems(Boolean isdeal, String category, List<String> rank, Integer level, String item, String beginDate, String endDate) {
        String datePattern = "\\d{4}-\\d{1,2}-\\d{1,2}";
        if (beginDate == null || endDate == null || !beginDate.matches(datePattern) || !endDate.matches(datePattern))
            return Result.error("日期格式有误");
        JSONArray jo = wnGkMsgService.getProblemListItems(isdeal, category, rank, level, item, beginDate + " 00:00:00", endDate + " 23:59:59");
        return Result.ok(jo);
    }

    public Result getStatisticalAnalysis(String beginDate, String endDate, Integer level, Integer orgId) {
        JSONObject jo = wnWellStatusService.getWellDataCount(endDate, level, orgId);
        JSONArray wellDataCount = jo.getJSONArray("data");
        if (wellDataCount != null) {
            wellDataCount = wnGkMsgService.getAllWnGkMsgCountByTime(beginDate, endDate, level, wellDataCount);//rank is 1,2,3
            wellDataCount = wnGkMsgService.getAllDealInTimeByTime(beginDate, endDate, level, wellDataCount);
            // feign 调用数据 计算安排措施落实率 实施措施及时率
            wellDataCount = JSONObject.parseArray(JSONObject.parseObject(
                    dgnsDataClient.getAllMeasureInfoAnalysis(beginDate, endDate, wellDataCount.toJSONString()), Result.class).getData().toString());
            wellDataCount.sort((x1, x2) -> {
                JSONObject obj1 = (JSONObject) x1;
                JSONObject obj2 = (JSONObject) x2;
                Double w1 = obj1.getDouble("BJCZL") * 4 + obj1.getDouble("WTCZL") * 3 + obj1.getDouble("APLSL") * 2 + obj1.getDouble("SSJSL");
                Double w2 = obj2.getDouble("BJCZL") * 4 + obj2.getDouble("WTCZL") * 3 + obj2.getDouble("APLSL") * 2 + obj2.getDouble("SSJSL");
                return w2.compareTo(w1);
            });
            for (int i = 0; i < wellDataCount.size(); i++) {
                wellDataCount.getJSONObject(i).put("rank", i + 1);
            }
            jo.put("data", wellDataCount);
        }
        return Result.ok(jo);
    }
}
