package com.gct.warning.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.common.core.result.Result;
import com.gct.warning.entity.WnGkMsg;
import com.gct.warning.entity.WnWellInfo;
import com.gct.warning.feignclient.BaseDataClient;
import com.gct.warning.mapper.WnGkMsgMapper;
import com.gct.warning.service.WnGkMsgService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.warning.service.WnWellInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.StreamUtils;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author xsr
 * @since 2021-05-07
 */
@Service
public class WnGkMsgServiceImpl extends ServiceImpl<WnGkMsgMapper, WnGkMsg> implements WnGkMsgService {

    @Autowired
    WnWellInfoService wnWellInfoService;
    @Autowired
    WnGkMsgMapper wnGkMsgMapper;
    @Autowired
    BaseDataClient baseDataClient;

    private Set<String> getIdSet(List<Map<String, Object>> mapList, String key) {
        HashSet<String> set = new HashSet<>();
        mapList.forEach(m -> {
            set.add(m.get(key).toString());
        });
        return set;
    }

    private List<String> getSetSameList(Set<String> s1, Set<String> s2) {
        ArrayList<String> list = new ArrayList<>();
        s1.forEach(x -> {
            if (s2.contains(x)) list.add(x);
        });
        return list;
    }

    public JSONArray getWnGkMsgCount(String date, Integer level, List<String> rank, JSONArray ja) {
        Set<String> wnIdSet = getIdSet(baseMapper.selectMaps(
                new QueryWrapper<WnGkMsg>().select("well_id").
                        eq("wn_date::date", LocalDate.parse(date))), "well_id");
        for (int i = 0; i < ja.size(); i++) {
            JSONObject jo = ja.getJSONObject(i);
            Integer orgId = jo.getInteger("Id");
            List<Map<String, Object>> wellIdList = wnWellInfoService.listMaps(new QueryWrapper<WnWellInfo>().
                    eq("level" + level, orgId).
                    select("well_id"));
            Set<String> wellIdSet = getIdSet(wellIdList, "well_id");
            List<String> sameList = getSetSameList(wnIdSet, wellIdSet);
            if (sameList.size() == 0) {
                jo.put("GKZS", 0);
                jo.put("GKWCL", 0);
                continue;
            }
            Integer notConfirmNum = baseMapper.selectCount(new QueryWrapper<WnGkMsg>().
                    eq("wn_date::date", LocalDate.parse(date)).
                    isNull("confirm").
                    in("rank", rank).
                    in("well_id", sameList));

            Integer confirmNum = baseMapper.selectCount(new QueryWrapper<WnGkMsg>().
                    isNotNull("confirm").
                    eq("wn_date::date", LocalDate.parse(date)).
                    in("rank", rank).
                    in("well_id", sameList)
            );

            jo.put("GKZS", notConfirmNum + confirmNum);
            jo.put("GKWCL", notConfirmNum);
        }
        return ja;
    }

    @Override
    public JSONArray getAllWnGkMsgCountByTime(String beginDate, String endDate, Integer level, JSONArray ja) {
        Set<String> wnIdSet = getIdSet(baseMapper.selectMaps(
                new QueryWrapper<WnGkMsg>().select("well_id").
                        between("wn_date::date", LocalDate.parse(beginDate), LocalDate.parse(endDate).plusDays(1))), "well_id");
        for (int i = 0; i < ja.size(); i++) {
            JSONObject jo = ja.getJSONObject(i);
            Integer orgId = jo.getInteger("Id");
            List<Map<String, Object>> wellIdList = wnWellInfoService.listMaps(new QueryWrapper<WnWellInfo>().
                    eq("level" + level, orgId).
                    select("well_id"));
            Set<String> wellIdSet = getIdSet(wellIdList, "well_id");
            List<String> sameList = getSetSameList(wnIdSet, wellIdSet);
            if (sameList.size() == 0) {
                jo.put("BJ", 0);
                jo.put("BJCZ", 0);
                jo.put("BJCZL", 0);
                continue;
            }
            Integer notConfirmNum = baseMapper.selectCount(new QueryWrapper<WnGkMsg>().
                    between("wn_date::date", LocalDate.parse(beginDate), LocalDate.parse(endDate).plusDays(1)).
                    isNull("confirm").
                    in("well_id", sameList));

            Integer confirmNum = baseMapper.selectCount(new QueryWrapper<WnGkMsg>().
                    isNotNull("confirm").
                    between("wn_date::date", LocalDate.parse(beginDate), LocalDate.parse(endDate).plusDays(1)).
                    in("well_id", sameList)
            );

            jo.put("BJ", notConfirmNum + confirmNum);
            jo.put("BJCZ", confirmNum);
            jo.put("BJCZL", notConfirmNum + confirmNum == 0 ? 0 : confirmNum * 1.0 / (confirmNum + notConfirmNum) * 100);
        }
        return ja;
    }

    @Override
    public JSONArray getAllDealInTimeByTime(String beginDate, String endDate, Integer level, JSONArray ja) {
        Set<String> wnIdSet = getIdSet(baseMapper.selectMaps(
                new QueryWrapper<WnGkMsg>().select("well_id").
                        between("wn_date::date", LocalDate.parse(beginDate), LocalDate.parse(endDate).plusDays(1))), "well_id");
        for (int i = 0; i < ja.size(); i++) {
            JSONObject jo = ja.getJSONObject(i);
            Integer orgId = jo.getInteger("Id");
            List<Map<String, Object>> wellIdList = wnWellInfoService.listMaps(new QueryWrapper<WnWellInfo>().
                    eq("level" + level, orgId).
                    select("well_id"));
            Set<String> wellIdSet = getIdSet(wellIdList, "well_id");
            List<String> sameList = getSetSameList(wnIdSet, wellIdSet);
            if (sameList.size() == 0) {
                jo.put("WTCZ", 0);
                jo.put("WTJS", 0);
                jo.put("WTCZL", 0);
                continue;
            }

            Integer confirmNum = baseMapper.selectCount(new QueryWrapper<WnGkMsg>().
                    isNotNull("confirm").
                    between("wn_date::date", LocalDate.parse(beginDate), LocalDate.parse(endDate).plusDays(1)).
                    in("well_id", sameList)
            );
            Integer inTimeConfirmNum = baseMapper.selectCount(new QueryWrapper<WnGkMsg>().
                    isNotNull("confirm").
                    between("wn_date::date", LocalDate.parse(beginDate), LocalDate.parse(endDate).plusDays(1)).
                    in("well_id", sameList).
                    apply("date_part('day',confirm_date::timestamp-wn_date::timestamp)<1", "")

            );
            jo.put("WTCZ", confirmNum);
            jo.put("WTJS", inTimeConfirmNum);
            jo.put("WTCZL", confirmNum == 0 ? 0 : inTimeConfirmNum * 1.0 / confirmNum * 100);
        }
        return ja;
    }

    @Override
    public String extractWorkingConditionDiagnosis(String wellId, String date) {
        SimpleDateFormat sdf = new SimpleDateFormat(" yyyy-MM-dd");
        if (StringUtils.isEmpty(date)) {
            date = sdf.format(new Date());
        }
        return wnGkMsgMapper.extractWorkingConditionDiagnosis(wellId, date);
    }


    @Override
    public JSONObject getAllGKWarning(String date, Integer level, List<String> rank, Integer orgId, Integer curPage, Integer pageSize) {
        IPage<WnGkMsg> page = baseMapper.getAllGKWarning(new Page<>(curPage, pageSize), date, level, rank, orgId);
        JSONObject jo = new JSONObject();
        jo.put("data", page.getRecords());
        jo.put("total", page.getTotal());
        return jo;
    }

    @Override
    public JSONObject getNotConfirmGKWarning(String date, Integer level, List<String> rank, Integer orgId, Integer curPage, Integer pageSize) {
        IPage<WnGkMsg> page = baseMapper.getNotConfirmGKWarning(new Page<>(curPage, pageSize), date, level, rank, orgId);
        JSONObject jo = new JSONObject();
        jo.put("data", page.getRecords());
        jo.put("total", page.getTotal());
        return jo;
    }

    @Override
    public void addUserConfirm(String problem, String
            measure, String username, String resultId) {
        WnGkMsg wnGkMsg = new WnGkMsg().setConfirm(problem).setNextStep(measure).setConfirmUser(username);
        System.out.println(wnGkMsg);
        baseMapper.update(wnGkMsg, new QueryWrapper<WnGkMsg>().eq("result_id", UUID.fromString(resultId)));
    }

    @Override
    public void addWnGkMsg(WnGkMsg wnGkMsg) {
        this.saveOrUpdate(wnGkMsg,new QueryWrapper<WnGkMsg>().eq("result_id",wnGkMsg.getResultId()));
    }

    @Override
    public JSONArray getProblemListItems(Boolean isdeal, String category, List<String> rank, Integer level, String paramArray, String beginDate, String endDate) {

        JSONArray args = JSONArray.parseArray(paramArray);//orgid、problem
        switch (category) {
            case "问题":
                return getProblemCategory(isdeal, category, rank, level, args.getInteger(0), args.size() > 1 ? args.getString(1) : null, beginDate, endDate);
            case "井区":
                return getLevelCategory(isdeal, category, rank, level, args.getInteger(0), beginDate, endDate, 2);
            case "站点":
                return getLevelCategory(isdeal, category, rank, level, args.getInteger(0), beginDate, endDate, 3);
            case "区块":
                return getLevelCategory(isdeal, category, rank, level, args.getInteger(0), beginDate, endDate, 3);
        }
        return null;
    }

    private Map<String, String> getCommonNameMap(String orgId) {
        //获取common_name
        JSONArray nameArray = JSONArray.parseArray(JSONObject.parseObject(baseDataClient.getJh(orgId), Result.class).getData().toString());
        Map<String, String> commonNameMap = new HashMap<>();
        for (int i = 0; i < nameArray.size(); i++) {
            commonNameMap.put(nameArray.getJSONObject(i).getString("id"), nameArray.getJSONObject(i).getString("value"));
        }
        return commonNameMap;
    }


    private JSONArray getLevelCategory(Boolean isdeal, String category, List<String> rank, Integer level, Integer orgId, String beginDate, String endDate, Integer num) {

        LocalDateTime begin = LocalDateTime.parse(beginDate,DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        LocalDateTime end = LocalDateTime.parse(endDate,DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        QueryWrapper<WnGkMsg> wrapper = new QueryWrapper<WnGkMsg>().between("wn_date", begin, end);
        wrapper = isdeal ? wrapper.isNotNull("confirm") : wrapper.isNull("confirm").in("rank", rank);
        List<WnGkMsg> wnGkMsgs = baseMapper.selectList(wrapper);
        Map<String, List<WnGkMsg>> gkMsgMap = new TreeMap<>();
        wnGkMsgs.forEach(x -> {
            if (gkMsgMap.containsKey(x.getWellId())) {
                gkMsgMap.get(x.getWellId()).add(x);
            } else {
                ArrayList<WnGkMsg> list = new ArrayList<>();
                list.add(x);
                gkMsgMap.put(x.getWellId(), list);
            }
        });
        JSONArray res = new JSONArray();
        if (level == 1) { //按大区查
            List<WnWellInfo> wnWellInfoList = wnWellInfoService.list(new QueryWrapper<WnWellInfo>().lt("spud_date", end));
            Map<String, Integer> countMap = new HashMap<>();
            Map<String, Integer> orgIdMap = new HashMap<>();
            wnWellInfoList.forEach(x -> countMap.put(x.getLevelName1(), 0));
            wnWellInfoList.forEach(x -> {
                List<WnGkMsg> wnGkMsgList = gkMsgMap.get(x.getWellId());
                if (wnGkMsgList != null && wnGkMsgList.size() > 0) {
                    Integer count = countMap.get(x.getLevelName1());
                    if (count == 0) {
                        orgIdMap.put(x.getLevelName1(), x.getLevel1());
                    }
                    countMap.put(x.getLevelName1(), count + wnGkMsgList.size());
                }
            });
            countMap.forEach((k, v) -> {
                JSONObject jo = new JSONObject();
                jo.put("context", k + "(" + v + ")");
                jo.put("orgId", orgIdMap.get(k));
                jo.put("level", 2);
                res.add(jo);
            });

        } else if (level == 2) {
            String colName1 = "level" + num;
            String colName2 = "level_name" + num;
            List<Map<String, Object>> wnWellInfoList = wnWellInfoService.listMaps(
                    new QueryWrapper<WnWellInfo>()
                            .select("well_id" + "," + colName1 + "," + colName2)
                            .eq("level1", orgId)
                            .lt("spud_date", end));
            Map<String, Integer> countMap = new HashMap<>();
            Map<String, Integer> orgIdMap = new HashMap<>();
            wnWellInfoList.forEach(m -> countMap.put(m.get(colName2).toString(), 0));
            wnWellInfoList.forEach(m -> {
                String wellId = m.get("well_id").toString();
                String levelId = m.get(colName1).toString();
                String levelName = m.get(colName2).toString();
                List<WnGkMsg> wnGkMsgList = gkMsgMap.get(wellId);
                if (wnGkMsgList != null && wnGkMsgList.size() > 0) {
                    Integer count = countMap.get(levelName);
                    if (count == 0) {
                        orgIdMap.put(levelName, Integer.parseInt(levelId));
                    }
                    countMap.put(levelName, count + wnGkMsgList.size());
                }
            });
            countMap.forEach((k, v) -> {
                JSONObject jo = new JSONObject();
                jo.put("context", k + "(" + v + ")");
                jo.put("orgId", orgIdMap.get(k));
                jo.put("level", 3);
                res.add(jo);
            });
        } else if (level == 3) {
            List<Map<String, Object>> wnWellInfoList = wnWellInfoService.listMaps(
                    new QueryWrapper<WnWellInfo>()
                            .select("well_id")
                            .eq("level" + num, orgId)
                            .lt("spud_date", end));
            //获取common_name
            Map<String, String> commonNameMap = getCommonNameMap(orgId.toString());
            wnWellInfoList.forEach(m -> {
                String wellId = m.get("well_id").toString();
                List<WnGkMsg> wnGkMsgList = gkMsgMap.get(wellId);
                if (wnGkMsgList != null && wnGkMsgList.size() > 0) {
                    wnGkMsgList.forEach(wnGkMsg -> {
                        /*//改 list
                        msgMap.put(wellId, wnGkMsg.getWellId() + " " + wnGkMsg.getContext());
                        resultIdMap.put(wellId, wnGkMsg.getResultId());*/
                        JSONObject jo = new JSONObject();
                        //需要name 而不是 well_id
                        jo.put("context", commonNameMap.get(wnGkMsg.getWellId()) + " " + wnGkMsg.getContext());
                        jo.put("resultId", wnGkMsg.getResultId());
                        jo.put("level", -1);
                        res.add(jo);
                    });
                }
            });
        }
        return res;
    }

    private JSONArray getProblemCategory(Boolean isdeal, String category, List<String> rank, Integer level, Integer orgId, String problem, String beginDate, String endDate) {
        LocalDateTime begin = LocalDateTime.parse(beginDate, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        LocalDateTime end = LocalDateTime.parse(endDate, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        QueryWrapper<WnGkMsg> wrapper = new QueryWrapper<WnGkMsg>().between("wn_date", begin, end);
        wrapper = isdeal ? wrapper.isNotNull("confirm") : wrapper.isNull("confirm").in("rank", rank);
        List<WnGkMsg> wnGkMsgs = baseMapper.selectList(wrapper);
        Map<String, WnGkMsg> gkMsgMap = new HashMap<>();
        wnGkMsgs.forEach(x -> gkMsgMap.put(x.getWellId(), x));
        JSONArray res = new JSONArray();
        if (level == 1) { //按大区查
            List<WnWellInfo> wnWellInfoList = wnWellInfoService.list(new QueryWrapper<WnWellInfo>().lt("spud_date", end));
            Map<String, Integer> countMap = new HashMap<>();
            Map<String, Integer> orgIdMap = new HashMap<>();
            wnWellInfoList.forEach(x -> countMap.put(x.getLevelName1(), 0));
            wnWellInfoList.forEach(x -> {
                WnGkMsg wnGkMsg = gkMsgMap.get(x.getWellId());
                if (wnGkMsg != null) {
                    Integer count = countMap.get(x.getLevelName1());
                    if (count == 0) {
                        orgIdMap.put(x.getLevelName1(), x.getLevel1());
                    }
                    countMap.put(x.getLevelName1(), count + 1);
                }
            });
            countMap.forEach((k, v) -> {
                JSONObject jo = new JSONObject();
                jo.put("context", k + "(" + v + ")");
                jo.put("orgId", orgIdMap.get(k));
                jo.put("level", 2);
                res.add(jo);
            });
        } else if (level == 2) {
            List<WnWellInfo> wnWellInfoList = wnWellInfoService.list(new QueryWrapper<WnWellInfo>().lt("spud_date", end).eq("level1", orgId));
            Map<String, Integer> countMap = new HashMap<>();
            wnWellInfoList.forEach(x -> {
                WnGkMsg wnGkMsg = gkMsgMap.get(x.getWellId());
                if (wnGkMsg != null) {
                    Integer count = countMap.get(wnGkMsg.getContext());
                    if (count == null) {
                        countMap.put(wnGkMsg.getContext(), 1);
                    } else countMap.put(wnGkMsg.getContext(), count + 1);
                }
            });
            countMap.forEach((k, v) -> {
                JSONObject jo = new JSONObject();
                jo.put("context", k + "(" + v + ")");
                jo.put("problem", k);
                jo.put("level", 3);
                jo.put("orgId", orgId);
                res.add(jo);
            });
        } else if (level == 3) {
            List<WnWellInfo> wnWellInfoList = wnWellInfoService.list(new QueryWrapper<WnWellInfo>().lt("spud_date", end).eq("level1", orgId));
            //获取common_name
            Map<String, String> commonNameMap = getCommonNameMap(orgId.toString());
            wnWellInfoList.forEach(x -> {
                WnGkMsg wnGkMsg = gkMsgMap.get(x.getWellId());
                if (wnGkMsg != null && wnGkMsg.getContext().equals(problem)) {
                    JSONObject jo = new JSONObject();
                    jo.put("context", commonNameMap.get(x.getWellId()) + ' ' + problem);
                    jo.put("problem", "无");
                    jo.put("level", -1);
                    jo.put("resultId", wnGkMsg.getResultId());
                    res.add(jo);
                }
            });
        }
        return res;
    }


}
