package com.gct.warning.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gct.warning.entity.WnWellInfo;
import com.gct.warning.entity.WnWellStatus;
import com.gct.warning.mapper.WnWellStatusMapper;
import com.gct.warning.service.WnWellInfoService;
import com.gct.warning.service.WnWellStatusService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author xsr
 * @since 2021-05-07
 */
@Service
public class WnWellStatusServiceImpl extends ServiceImpl<WnWellStatusMapper, WnWellStatus> implements WnWellStatusService {

    @Autowired
    public WnWellInfoService wnWellInfoService;

    @Override
    public void saveAndUpdate(List<WnWellStatus> wnWellStatusList) {
        wnWellStatusList.forEach(x -> {
            Map<String, Object> conditionMap = new HashMap<>();
            conditionMap.put("well_id", x.getWellId());
            conditionMap.put("prod_date", x.getProdDate());
            QueryWrapper<WnWellStatus> wrapper = new QueryWrapper<WnWellStatus>().allEq(conditionMap);
            WnWellStatus wnWellStatus = this.getOne(wrapper);
            if (wnWellStatus != null) {
                if (!x.equals(wnWellStatus)) {
                    this.update(x, wrapper);
                }
            } else {
                this.save(x);
            }
        });
    }

    @Override
    public JSONObject getWellDataCount(String endDate, Integer level, Integer orgId) {
        Map<Integer, Integer> allNum = new HashMap<>();
        Map<Integer, Integer> closeNum = new HashMap<>();
        Map<Integer, String> idName = new HashMap<>();
        QueryWrapper<WnWellStatus> wrapper = new QueryWrapper<WnWellStatus>().
                eq("prod_date::date", LocalDate.parse(endDate).plusDays(-1)).   // endDate-1 统计开关井数量
                select("level_name" + level + ",count(*),level" + level).
                groupBy("level_name" + level + ",level" + level);
        if (orgId != 0) wrapper = wrapper.eq("level" + (level - 1), orgId); //以当前orgId为标准过滤数据
        List<Map<String, Object>> mapList = baseMapper.selectMaps( wrapper);
        if (!(mapList == null || mapList.size() == 0 || mapList.get(0).get("level_name" + level) == null)){ //当天有数据执行if，无数据后续会填充默认0数据
            mapList.forEach(x -> {
                Integer id = Integer.parseInt(x.get("level" + level).toString());
                String name = x.get("level_name" + level).toString();
                Integer count = Integer.parseInt(x.get("count").toString());
                allNum.put(id, count);
                idName.put(id, name);
            });
            wrapper = new QueryWrapper<WnWellStatus>().
                    eq("prod_date::date", LocalDate.parse(endDate)).
                    eq("prod_time", 0).
                    select("level_name" + level + ",count(*),level" + level).
                    groupBy("level_name" + level + ",level" + level);
            if (orgId != 0) wrapper = wrapper.eq("level" + (level - 1), orgId); //以当前orgId为标准过滤数据
            mapList = this.listMaps(wrapper);
            mapList.forEach(x -> {
                Integer id = Integer.parseInt(x.get("level" + level).toString());
                Integer count = Integer.parseInt(x.get("count").toString());
                closeNum.put(id, count);
            });
        }
        JSONArray ja = new JSONArray();
        allNum.forEach((k, v) -> {
            JSONObject jo = new JSONObject();
            Integer close = closeNum.get(k);
            Integer open = v - (close == null ? 0 : close);
            jo.put("Id", k);
            jo.put("JHKJS", v);
            jo.put("SJKJS", open);
            jo.put("TJS", close == null ? 0 : close);
            jo.put("ZYQ", idName.get(k));
            ja.add(jo);
        });
        // 开井 停井等信息是按照prod date 查询得到的 按照范围选取时可能会因为没有数据而影响后面列数据的查询
        // 加入默认的0数据，提供给后续列数据查询使用org信息
        List<Map<String, Object>> wellInfoList =wnWellInfoService.listMaps(
                new QueryWrapper<WnWellInfo>().select("DISTINCT level_name" + level + ",level" + level).isNotNull("level"+level).lt("spud_date", LocalDate.parse(endDate)));
        System.out.println(wellInfoList);
        wellInfoList.forEach(m->{
            Integer id = Integer.parseInt(m.get("level" + level).toString());
            String name = m.get("level_name" + level).toString();
            if(!allNum.containsKey(id)){ //不存在数据时加入默认0数据
                JSONObject jo = new JSONObject();
                jo.put("Id", id);
                jo.put("JHKJS", 0);
                jo.put("SJKJS", 0);
                jo.put("TJS", 0);
                jo.put("ZYQ", name);
                ja.add(jo);
            }
        });
        JSONObject jo = new JSONObject();
        jo.put("data", ja);
        return jo;
    }
}
